use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006000
eval "use Test::Spelling 0.12; use Pod::Wordlist::hanekomu; 1" or die $@;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
BENCHMARKING
CRLF
SHA
canonpath
codepoints
cwd
dirname
lstat
mkpath
opena
openr
openrw
openw
realpath
stringifying
subclasses
touchpath
unlinked
utf
David
Golden
dagolden
Chris
Williams
bingos
Steinbrunner
dsteinbrunner
Gabor
Szabo
szabgab
George
Hartzell
hartzell
Goro
Fuji
gfuji
Karen
Etheridge
ether
Keedi
Kim
keedi
Michael
Schwern
mschwern
lib
Path
Tiny
